package dev.kske.chess.board;

import java.util.Objects;

/**
 * Project: <strong>Chess</strong><br>
 * File: <strong>Move.java</strong><br>
 * Created: <strong>02.07.2019</strong><br>
 * 
 * @since Chess v0.1-alpha
 * @author Kai S. K. Engelbart
 */
public class Move {

	public final Position	pos, dest;
	public final int		xDist, yDist, xSign, ySign;
	public Type				type;

	public Move(Position pos, Position dest, Type type) {
		this.pos	= pos;
		this.dest	= dest;
		this.type	= type;
		xDist		= Math.abs(dest.x - pos.x);
		yDist		= Math.abs(dest.y - pos.y);
		xSign		= (int) Math.signum(dest.x - pos.x);
		ySign		= (int) Math.signum(dest.y - pos.y);
	}

	public Move(Position pos, Position dest) {
		this(pos, dest, Type.NORMAL);
	}

	public Move(int xPos, int yPos, int xDest, int yDest) {
		this(new Position(xPos, yPos), new Position(xDest, yDest));
	}

	public static Move fromLAN(String move) {
		return new Move(Position.fromLAN(move.substring(0, 2)),
				Position.fromLAN(move.substring(2)));
	}

	public String toLAN() {
		return pos.toLAN() + dest.toLAN();
	}

	public boolean isHorizontal() { return yDist == 0; }

	public boolean isVertical() { return xDist == 0; }

	public boolean isDiagonal() { return xDist == yDist; }

	@Override
	public String toString() {
		return String.format("%s -> %s", pos, dest);
	}

	@Override
	public int hashCode() {
		return Objects.hash(dest, pos, type, xDist, xSign, yDist, ySign);
	}

	@Override
	public boolean equals(Object obj) {
		if (this == obj) return true;
		if (obj == null) return false;
		if (getClass() != obj.getClass()) return false;
		Move other = (Move) obj;
		return Objects.equals(dest, other.dest) && Objects.equals(pos, other.pos) && type == other.type
				&& xDist == other.xDist && xSign == other.xSign && yDist == other.yDist && ySign == other.ySign;
	}

	public static enum Type {
		NORMAL, PAWN_PROMOTION, CASTLING, EN_PASSANT, UNKNOWN
	}
}
